@@ -1,4 +1,10 @@
 """semopy: Structural Equation Modeling Optimization in Python"""
+
+name = "semopy"
+__version__ = "2.2.0"
+__author__ = "Georgy Meshcheryakov"
+
+
 from .model_generalized_effects import ModelGeneralizedEffects
 from .model_effects import ModelEffects
 from .model_means import ModelMeans
@@ -7,11 +13,12 @@ from .stats import calc_stats, gather_statistics
 from .means import estimate_means
 from .plot import semplot
 from .model import Model
+from .unbias import bias_correction
+from .optimizer import Optimizer
+from .report import report
+from . import model_generation
 from . import effects 
 from . import multigroup
 from . import examples
+from . import utils
 from . import efa
-
-name = "semopy"
-__version__ = "2.1.7"
-__author__ = "Georgy Meshcheryakov"
\ No newline at end of file
